package GJ::Vite;
use utf8;
use warnings;
use strict;

use GJ::Boot 'VITE';
our $VERSION = '0.500';

sub Import {
    my $userclass = shift;
}

sub Init {
    my $Module = shift;
    my $args = shift || {};
    my %activate;
    if('HASH' eq ref $args) {
        while (my ($key, $conf) = each %$args)  {
            if('HASH' eq ref $conf) {
                if(my $tok = connectKit($Module, $key, $conf)) {
                    $Module->Log(info => "GJVite: Will handle '$key' with '$tok'");
                    $activate{$key} = $tok;
                }
            } else {
                $Module->Log(warn => "GJVite: ignored '$key' malformed  '$conf'!")
            }
        }
    } else {
        $Module->Log(warn => "GJVite: malformed  $args!")
    }
    GJ::FWServices::PostDispatchReg($Module, sub {
        # Tobe executed once per request after Module's Work
        my($Module, $r, $action) = @_;
        my $res = $action;
        unless($Module->Attended || ref($action) ||
               Scalar::Util::looks_like_number($res)
        ) {
            my $tok = $activate{$action};
            if($tok) {
                warn "V: $action $tok\n";
                $Module->SendRedirect($activate{$action})
            }
        }
        return undef;
    });
    1;
}

sub connectKit {
    my $Module = shift;
    my $key = shift;
    my $conf = shift;
    my $mode = $conf->{Mode} || 'static';
    my $mbase = $Module->MagicBase;
    my $path = $conf->{Path};
    my $base = $conf->{Base} || $Module->Param("Vite_${key}_Base") || $key;
    $base = "/$base" unless $base =~ /^\//;
    $Module->Log(info => "GJVite: Connect '$key' $base $path");
    if($mode eq 'static') {
        # Prepare static content in $base
        $Module->Log(info => "GJVite: Configuring static $key...");
        if(!-f "$path/index.html") {
            $Module->Log(warn => "No fallback ('index.html') in '$key'");
        }
        $Module->AddApacheConf( qq|Alias $base $path| );
        return $base;
    } elsif($mode eq 'proxy') {
        $Module->Log(info => "GJVite: Configuring proxy $key");
        my $port = $conf->{Port} || $Module->Param("Vite_${key}_Port") || 4173;
        my $selfsvr = "localhost:80";
        $selfsvr =~ s/:\d+/':' . $port/e;
        $Module->Log(info => "GJVite PB: $selfsvr/$base");
        $Module->AddApacheConf(
            qq|ProxyPass "/$base" "http://$selfsvr/$base" upgrade=websocket|
        );
        return undef; # No ready yet
    } else {
        $Module->Log(warn => "Vite: unknow mode $mode, ignored $base")
    }
    return undef;
}


